"
    Copyright (c) 2016-now, Jan Vrany <jan . vrany [at] fit . cvut . cz>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, 
    are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, 
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice, 
       this list of conditions and the following disclaimer in the documentation 
       and/or other materials provided with the distribution.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS IS' AND 
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
    DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
    ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
    ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"
"{ Package: 'jv:dragonfly/udis86sx' }"

"{ NameSpace: Smalltalk }"

Object subclass:#UDIS86Instruction
	instanceVariableNames:'pc len assembly mnemonic operand0 operand1 operand2'
	classVariableNames:''
	poolDictionaries:'UDIS86Types UDIS86Mnemonics'
	category:'UDIS86'
!

!UDIS86Instruction class methodsFor:'documentation'!

copyright
"
    Copyright (c) 2016-now, Jan Vrany <jan . vrany [at] fit . cvut . cz>
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, 
    are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, 
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice, 
       this list of conditions and the following disclaimer in the documentation 
       and/or other materials provided with the distribution.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS IS' AND 
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
    DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
    ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
    ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"
! !

!UDIS86Instruction methodsFor:'accessing'!

address
    "Return PC of this instruction. This if for compatibility with 
     GDBInstruction."

    ^ pc

    "Created: / 25-06-2018 / 17:17:02 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 26-06-2018 / 11:29:40 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

assembly
    ^ assembly
!

branchTarget
    "
     Return and address of next instruction if branch is taken.
     Assumes that this instruction is a branch instruction."
    
    self assert:self isBranch.
    self assert:operand0 notNil.
    operand0 type = UD_OP_JIMM ifTrue:[
        ^ pc + len + operand0 sval.
    ].
    operand0 type = UD_OP_REG ifTrue:[
        "/ jump is to an address in register, hence
        "/ not statically known.
        ^ nil 
    ].
    operand0 type = UD_OP_MEM ifTrue:[
        "/ jump is to an address in in memory.
        "/ not statically known.
        ^ nil 
    ].
    self assert: false.

    "Created: / 26-06-2018 / 11:38:34 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 08-05-2019 / 13:07:02 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!UDIS86Instruction methodsFor:'enumerating'!

instructionsDo: aBlock
    aBlock value: self.

    "Created: / 16-08-2018 / 16:15:05 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!UDIS86Instruction methodsFor:'initialization'!

setAssembly: aString
    assembly := aString.

    "Created: / 10-12-2015 / 17:04:43 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

setLength:anInteger 
    len := anInteger

    "Created: / 25-06-2018 / 15:34:59 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

setMnemonic:anInteger 
    mnemonic := anInteger

    "Created: / 10-12-2015 / 21:12:58 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

setOperand0: anUDIS86Operand
    operand0 := anUDIS86Operand

    "Created: / 25-06-2018 / 15:36:05 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

setOperand1: anUDIS86Operand
    operand1 := anUDIS86Operand

    "Created: / 25-06-2018 / 15:35:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

setOperand2: anUDIS86Operand
    operand2 := anUDIS86Operand

    "Created: / 25-06-2018 / 15:36:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

setPC: anInteger
    pc := anInteger

    "Created: / 10-12-2015 / 16:55:40 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!UDIS86Instruction methodsFor:'printing'!

printOn: aStream
    pc printOn: aStream base: 16 size: ExternalAddress pointerSize * 2 fill: $0.
    aStream space.
    aStream nextPutAll: assembly

    "Created: / 11-12-2015 / 21:30:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!UDIS86Instruction methodsFor:'testing'!

isBranch
    "
     Return `true` is this instruction is some sort of branch instruction."
    
    ^ mnemonic between:UD_Ijo and:UD_Ijmp

    "Created: / 26-06-2018 / 11:35:57 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isReturn
    "Return `true` is this instruction is a return"
    
    ^ mnemonic between:UD_Iret and:UD_Iretf

    "Created: / 26-06-2018 / 15:39:00 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 16-08-2018 / 14:02:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!UDIS86Instruction class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

